from datetime import datetime as dt

import peewee
from bw2data import config, projects
from bw2data.sqlite import SubstitutableDatabase


class CommentThread(peewee.Model):
    resolved = peewee.BooleanField(default=False)
    name = peewee.TextField(null=False)
    created = peewee.DateTimeField(default=dt.utcnow)
    modified = peewee.DateTimeField(default=dt.utcnow)
    process_id = peewee.IntegerField(null=False)

    def __len__(self):
        return Comment.select().where(Comment.thread == self.id).count()

    def comments(self):
        return (
            Comment.select()
            .where(Comment.thread == self.id)
            .order_by(Comment.position.asc())
        )

    @property
    def reporter(self):
        try:
            min_position = (
                Comment.select(peewee.fn.Min(Comment.position))
                .where(Comment.thread == self)
                .scalar()
            )
            return Comment.get(position=min_position, thread=self).user
        except Exception as err:
            print("Error:", err)
            raise err

    def next_position(self):
        if not len(self):
            return 0
        else:
            return (
                Comment.select(peewee.fn.Max(Comment.position))
                .where(Comment.thread == self.id)
                .scalar()
            ) + 1

    class Meta:
        constraints = [peewee.SQL("UNIQUE (process_id, name)")]


class Comment(peewee.Model):
    thread = peewee.ForeignKeyField(CommentThread)
    position = peewee.IntegerField(null=False)
    content = peewee.TextField(null=False)
    # Don't want to bother with normalizing and CRUD for users
    # which are defined separate in app config anyway
    user = peewee.TextField(null=False)

    class Meta:
        constraints = [peewee.SQL("UNIQUE (thread_id, position)")]

    def save(self, *args, **kwargs):
        if not self.id:
            self.position = self.thread.next_position()
        obj = super().save(*args, **kwargs)
        CommentThread.update(modified=dt.utcnow()).where(
            CommentThread.id == self.thread_id
        ).execute()
        return obj


comments_db = SubstitutableDatabase(
    projects.dir / "comment_threads.db",
    [CommentThread, Comment],
)
config.sqlite3_databases.append(
    (
        "comment_threads.db",
        comments_db,
    )
)



from pathlib import Path
import csv
from bw_processing import safe_filename

import bw2data as bd
from bw2data.backends import Activity

from .matching import BASE_MATCH_TYPES


def get_csv_row(proxy: Activity) -> dict:
    orig = bd.get_node(id=proxy["original_id"])

    return {
        "source_name": orig.get("name"),
        "source_reference_product": orig.get("reference product"),
        "source_unit": orig.get("unit"),
        "source_location": orig.get("location"),
        "source_database": orig["database"],
        "target_name": proxy.get("name"),
        "target_reference_product": proxy.get("reference product"),
        "target_unit": proxy.get("unit"),
        "target_location": proxy.get("location"),
        "target_database": proxy["database"],
        "comment": proxy.get("comment"),
        "match_type": BASE_MATCH_TYPES.get(proxy.get("match_type")),
    }


def export_database_to_csv(db_name: str, directory: Path):
    COLUMNS = ["name", "reference_product", "unit", "location", "database"]
    HEADERS = (
        ["source_{}".format(x) for x in COLUMNS]
        + ["target_{}".format(x) for x in COLUMNS]
        + ["comment", "match_type"]
    )

    filename = safe_filename(db_name) + ".csv"
    fp = directory / filename

    with open(fp, "w") as f:
        writer = csv.DictWriter(f, HEADERS)
        writer.writeheader()
        for process in bd.Database(db_name):
            writer.writerow(get_csv_row(process))

    return fp



"""bw_matchbox."""
__all__ = (
    "__version__",
    "configure_app",
    "matchbox_app",
)

from .utils import get_version_tuple

__version__ = get_version_tuple()

from .webapp import configure_app, matchbox_app
